<?php
namespace Drupal\simple_taxonomy_menu\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\taxonomy\Entity\Vocabulary;

use Drupal\Core\Database\Database;
use Kint\Kint;

/**
 * Provides a 'Hello' Block.
 *
 * @Block(
 *   id = "simple_taxonomy_menu_block",
 *   admin_label = @Translation("Simple Taxonomy Menu block"),
 *   category = @Translation("Simple taxonomy Menu"),
 * )
 */
class SimpleTaxonomyMenuBlock extends BlockBase {

  private $routeMatch = null;
  private $connection = null;

  public function __construct() {
    $this->routeMatch = \Drupal::routeMatch();
    $this->connection = Database::getConnection();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    return [
      '#markup' => $this->simple_taxonomy_menu_createmenu(),
      '#attached' => [
        'library' => ['simple_taxonomy_menu/block-menu'],
      ],
    ];
  }

  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $vocabs = Vocabulary::loadMultiple();

    $list = [];
    foreach($vocabs as $vid => $vocab){
      $list[$vid] = $vocab->get('name');
    }

    $form['simple_taxonomy_menu_vid'] = [
      '#type' => 'select',
      '#title' => $this->t('Choose a taxonomy to create a menu out of'),
      '#default_value' => (string) $config['simple_taxonomy_menu_vid'],
      '#options' => $list,
    ];

    $form['simple_taxonomy_menu_before'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Menu items above'),
      '#description' => $this->t('Write down urls you want in the top of the menu.<br />'
        . 'Write down urls in as follow: url;title<br />'
        . 'Example external: http://www.google.com;Google<br />'
        . 'Example internal: node;Node list'),
      '#default_value' => isset($config['simple_taxonomy_menu_before']) ? $config['simple_taxonomy_menu_before'] : '',
    ];

    $form['simple_taxonomy_menu_after'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Menu items below'),
      '#description' => $this->t('Write down urls you want in the bottom of the menu.<br />'
        . 'Write down urls in as follow: url;title<br />'
        . 'Example external: http://www.google.com;Google<br />'
        . 'Example internal: node;Node list'),
      '#default_value' => isset($config['simple_taxonomy_menu_after']) ? $config['simple_taxonomy_menu_after'] : '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['simple_taxonomy_menu_vid'] = $values['simple_taxonomy_menu_vid'];
    $this->configuration['simple_taxonomy_menu_before'] = $values['simple_taxonomy_menu_before'];
    $this->configuration['simple_taxonomy_menu_after'] = $values['simple_taxonomy_menu_after'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {

  }

  /**
   * Gets menu items and creates an html list.
   *
   * @param int $vid
   *  Taxonomy vocab id.
   * @param string $delta
   *  Block delta name.
   *
   * @return string
   *  Returns html string with menu in it.
   */
  public function simple_taxonomy_menu_createmenu() {
    $config = $this->getConfiguration();


    if (empty($config['simple_taxonomy_menu_vid'])) {
      return '';
    }

    $vid = $config['simple_taxonomy_menu_vid'];
    $linksBefore = !empty($config['simple_taxonomy_menu_before']) ? $config['simple_taxonomy_menu_before'] : '';
    $linksAfter = !empty($config['simple_taxonomy_menu_before']) ? $config['simple_taxonomy_menu_before'] : '';
    $items = $this->simple_taxonomy_menu_loadmenu($vid, $linksBefore, $linksAfter);

    $html = $this->simple_taxonomy_menu_createlist($items, '0');

    return $html;
  }

  /**
   * Create html menu list.
   *
   * @param array $items
   *  An array of items that needs to be in the menu.
   * @param int $parent
   *  The parent level of the menu items that needs to be proccesed.
   *
   * @return string
   *  Returns the menu with all menu item layers.
   */
  public function simple_taxonomy_menu_createlist($items, $parent) {

    $li = '';
    $first = true;
    $lastTid = $this->simple_taxonomy_menu_last_item($items, $parent);
    foreach ($items as $key => $item) {

      if(empty($item['term'])){
        continue;
      }
      $term = $item['term'];

      if ($parent == '0' && is_string($term)) {
        $activeClass = '';
        $class = '';
        if ($first == true) {
          $class = 'first';
          $first = false;
        }
        if(\Drupal::service('path.current')->getPath()==$key){
          $class.= ' active-trail';
          $activeClass = 'active-trail active';
        }
        $li.= '<li class="menu__item is-leaf ' . $class . ' leaf" ><a title="menu__link ' . t($term) . '" class="' . $activeClass . '" href="' . Url::fromUri($key)->toString() . '">' . t($term) . '</a></li>'."\n";
      } else {

        if (!empty($term) && !empty($term->parents) && $term->parents[0] == $parent) {

          $activeClass = '';
          $class = '';
          if ($first == true) {
            $class = 'first';
            $first = false;
          } elseif ($lastTid == $term->tid) {
            $class = 'last';
          }



          if ($item['active'] == '1') {
            $class.= ' active-trail';
            $activeClass = 'active-trail active';
          }

          $links = $this->simple_taxonomy_menu_createlist($items, $term->tid);

          if ($links) {
            if ($item['open'] == '1') {
              $class.= ' expanded active-trail';
              $activeClass = 'active-trail active';
            } else {
              $class.= ' collapsed';
            }
            $class.= ' expendable';
          }
          $li.= '<li class="menu__item is-leaf ' . $class . '" ><a title="' . t($term->name) . '" class="menu__link ' . $activeClass . '" href="' . Url::fromUri('internal:/taxonomy/term/' . $term->tid)->toString() . '">' . t($term->name) . '</a>'
            . $links
            . '</li>'."\n";
        }
      }
    }


    if (!empty($li)) {
      $html = '<ul id="category-menu-' . $parent . '" class="simple-taxonomy-menu menu clearfix">'."\n".'' . $li . ''."\n".'</ul>';
      return $html;
    } else {
      return '';
    }
  }

  /**
   * Get the last item in the menu list.
   *
   * @param array $items
   *  List of menu items.
   * @param type $parent
   *  The parent level to show menu.
   *
   * @return int
   *  Returns tid of the last item in the menu.
   */
  public function simple_taxonomy_menu_last_item($items, $parent) {
    $tid = null;

    foreach ($items as $item) {
      if(!empty($item['term'])){
        $term = $item['term'];
        if (!empty($term->tid) && $term->parents[0] == $parent) {
          $tid = $term->tid;
        }
      }
    }

    return $tid;
  }

  /**
   * Load menu with all extra menu items.
   *
   * @param int $vid
   *  Taxonomy vocab id.
   * @param string $delta
   *  Block delta name.
   *
   * @return array
   *  Returns an array of all menu items.
   */
  public function simple_taxonomy_menu_loadmenu($vid, $linksBefore = '', $linksAfter = '') {

    $terms = \Drupal::service('entity_type.manager')
      ->getStorage("taxonomy_term")
      ->loadTree($vid, $parent = 0, $max_depth = NULL, $load_entities = FALSE);
    //$terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vid);//taxonomy_get_tree($vid);


//    $arg0 = arg(0);
//    $arg1 = NULL;
//    $arg2 = NULL;
    //d($this->routeMatch->getRawParameters());
    $node = $this->routeMatch->getParameter('node');
    $taxonomy = $this->routeMatch->getParameter('taxonomy_term');

    if(empty($_SESSION['simple_taxonomy_menu_active'])
    || (empty($node) && empty($taxonomy))){
      $_SESSION['simple_taxonomy_menu_active'] = NULL;
    }

//    if ($arg0 != 'taxonomy' && $arg0 != 'node') {
//      $_SESSION['simple_taxonomy_menu_active'] = NULL;
//    }

//    if ($arg0 == 'taxonomy') {
//      $arg1 = arg(1);
//      if ($arg1 == 'term') {
//        $arg2 = arg(2);
//      }
//    }

    // If user is viewing node we need to check if we still got a path
    if ($node) {

      $sql = $this->connection->select('taxonomy_index', 'ti');
      $sql->join('taxonomy_term_data', 'ttd', 'ti.tid = ttd.tid');
      $sql->fields('ti', array('tid'));
      $sql->condition('ti.nid', (int) $node->nid->getString(), '=');
      $sql->condition('ttd.vid', (int) $vid, '=');

      $results = $sql->execute();
      $node_terms = array();

      while ($result = $results->fetchAssoc()) {
        $node_terms[$result['tid']] = $result['tid'];
      }

      // Check if the active term is also present in the currently viewed node.
      if (!empty($_SESSION['simple_taxonomy_menu_active']) && !empty($node_terms[$_SESSION['simple_taxonomy_menu_active']])) {
        // Current term is in the node so all is good.
      } else {
        // The current term isn't in the current node, we gonna get the first term from the node.
        if (!empty($node_terms) && is_array($node_terms)) {
          // Get first values of the array
          foreach($node_terms as $node_term){
            $_SESSION['simple_taxonomy_menu_active'] = $node_term;
            break;
          }

        } else {
          $_SESSION['simple_taxonomy_menu_active'] = NULL;
        }
      }
    }


    $items = array();

    $links_before = $linksBefore;
    $ar_list_before = explode(PHP_EOL, $links_before);

    foreach($ar_list_before as $link_string){
      if(!empty($link_string)){
        $ar_link_before = explode(';', $link_string);
        $items[$ar_link_before[0]]['term'] = trim($ar_link_before[1]);
      }
    }

    foreach ($terms as $term) {

      $items[$term->tid]['term'] = $term;
      $items[$term->tid]['active'] = 0;
      $items[$term->tid]['open'] = 0;

      if ($taxonomy) {
        if ($taxonomy->tid->getString() == $term->tid) {
          $items[$term->tid]['active'] = 1;

          $_SESSION['simple_taxonomy_menu_active'] = $term->tid;
        }
      } elseif ($node) {
        if (!empty($_SESSION['simple_taxonomy_menu_active']) && $_SESSION['simple_taxonomy_menu_active'] == $term->tid) {
          $items[$term->tid]['active'] = 1;
        }
      }
    }

    $links_after = $linksAfter;
    $ar_list_after = explode(PHP_EOL, $links_after);

    foreach($ar_list_after as $link_string){
      if(!empty($link_string)){
        $ar_link_after = explode(';', $link_string);
        $items[$ar_link_after[0]]['term'] = trim($ar_link_after[1]);
      }
    }

    $items = $this->simple_taxonomy_menu_open_parent($items, $_SESSION['simple_taxonomy_menu_active']);

    return $items;
  }

  /**
   * Set the active parents.
   *
   * @param array $items
   *  Menu list to ge through.
   * @param int $tid
   *  Term id to calculate the active path.
   *
   * @return array
   *  Returns the menu array with terms set to open.
   */
  public function simple_taxonomy_menu_open_parent($items, $tid) {
    $items[$tid]['open'] = 1;
    if (!empty($items[$tid]['term']->parents) && $items[$tid]['term']->parents[0] != '0' && !empty($items[$tid]['term']->parents[0])) {
      $items = $this->simple_taxonomy_menu_open_parent($items, $items[$tid]['term']->parents[0]);
    }
    return $items;
  }

}